<?php

    class In8_NetSuite_Inventory {

        public static $optionsTag = 'in8_netsuite_inventory';

        public static function getOptionsTag() {
            return self::$optionsTag;
        }

        public function __construct() {
            $options = self::options();

            if (isset($options['enable_inventory_sync']) && $options['enable_inventory_sync'] == 'on') {
                
                add_action('init', [$this, 'process_inventories']);
                add_action('init', [$this, 'get_inventories']);
                add_action('in8_netsuite_process_inventories', [$this, 'process_inventories_action']);
            }

            //tab link and content
            add_filter('in8_admin_tab', [$this, 'addTab']);
            add_action('in8_netsuite_top_level_tabs_page', [$this, 'tabContent']);

            add_action('wp_ajax_' . self::getOptionsTag(), [$this, 'save_form']);

            // Ajax for getting data from NetSuite
            add_action('wp_ajax_In8_Inventory_NS', [$this, 'getInternalIdsFromNS']);
        }

        public function addTab($tabs) {
            $tabs[self::$optionsTag] = ['label' => "Inventory", "priority" => 1];
            return $tabs;
        }

        public function tabContent() {

            $options = In8_NetSuite_Inventory::getOptions();

            if (isset($options['locations']) && !empty($options['locations'])) {
                $locations = $options['locations'];
            }
            if (isset($options['subsidiaries']) && !empty($options['subsidiaries'])) {
                $subsidiaries = $options['subsidiaries'];
            }

            require_once('page-content.php');
        }

        public static function save_form() {

            $options = $_REQUEST['options'];

            update_option(self::getOptionsTag(), $options);

            die(json_encode([
                'type' => 'success',
            ]));
        }

        // /frontSide



        public static function options() {
            return get_option(self::getOptionsTag());
        }

        static function process_inventories() {
            if (!isset($_REQUEST['process_inventories'])) {
                return FALSE;
            }

            self::process_inventories_action();
            echo "Ok";
            exit;
        }

        static function get_inventories() {
            if (!isset($_REQUEST['get_inventories'])) {
                return FALSE;
            }

            echo "<pre>";
            print_r(self::getInventoriesFromNetsuite());
            echo "</pre>";

            exit;
        }

        static function process_inventories_action() {
            $options = self::options();
            
            $netsuite_connection = self::getNetsuiteConnection();
			$netsuite_connection->processMessage('Inventory update start.');
            
            $productsInWp = self::getProductsByInternalId();
            if (empty($productsInWp)) {
                echo "Inventory Sync: getProductsByInternalId returned empty set";
				$netsuite_connection->processMessage('<strong>getProductsByInternalId returned empty set</strong>', true, true);
				return false;
            }
            $inventories = self::getInventoriesFromNetsuite();


            if (empty($inventories)) {
                echo "getInventoriesFromNetsuite returned empty set";
				$netsuite_connection->processMessage('<strong>getInventoriesFromNetsuite returned empty set</strong>', true, true);
				return false;
            }

            $map = self::getLocationMap();
	
			// count the products inside the $productsInWp array and add a message explaining how many products will be updated.
			$wc_products = array_sum(array_map(function($items){ return count($items); }, $productsInWp));
			$netsuite_connection->processMessage("{$wc_products} WC products will be updated.");
	
			$request_stats = ['updated' => 0, 'inventories_missing' => 0, 'invalid_products'];
			foreach($productsInWp as $_internalId => $posts){
				foreach ($posts as $post) {
					$internalId = $_internalId;
					
					$product = wc_get_product($post->ID);
					
					if (!array_key_exists($internalId, $inventories)) {
						$request_stats['inventories_missing'] ++;
						continue;
					}
					
					$inventory = $inventories[$internalId];
					
					if (empty($inventory)) {
						$request_stats['inventories_missing'] ++;
						continue;
					}
					
					//$quantity = $inventory['locations'][0]['quantity'];
					
					$theid = $post->ID;
					//$sku = get_post_meta($theid, '_sku', true );
					$ptype = get_post_type($theid);
					$stockst = get_post_meta($theid, '_stock_status', true);
					
					$pststatus = get_post_status($post->ID);
					//echo '<br/><br /> --- <br />';
					if ($pststatus == 'trash') {
						$request_stats['invalid_products'] ++;
						//echo $pststatus .'<br/><br /> --- <br />';
						continue;
					}
					$posttype = $product->get_type();
					if ($posttype == 'bundle') {
						$request_stats['invalid_products'] ++;
						continue;
					}
					$parent_id = wp_get_post_parent_id($post->ID);
					
					if ($posttype == 'variation') {
						$parent_status = get_post_status($parent_id);
						if ($parent_status == '') {
							$request_stats['invalid_products'] ++;
							continue;
						}
						if ($parent_id == 0) {
							$request_stats['invalid_products'] ++;
							continue;
						}
						if( get_post_type($parent_id) != 'product' ){
							$request_stats['invalid_products'] ++;
							continue;
						}
					}
					
					if ($posttype == '') {
						$request_stats['invalid_products'] ++;
						continue;
					}
					
					if (isset($options['use_all_location_inventory']) && $options['use_all_location_inventory'] == 'on') {
						$quantity = $inventory['quantity'];
						
						update_post_meta($post->ID, "_stock", $quantity);
						
						$stockStatus = 'outofstock';
						if ($quantity > 0) {
							$stockStatus = 'instock';
						}
						if ($options['backorders'] != 'disable') {
							// backorders
							update_post_meta($post->ID, '_backorders', $options['backorders']);
							if( in_array($options['backorders'], ['yes', 'notify']) ){
								$stockStatus = 'instock';
							}
						}
						
						if ($posttype == 'variable') {
							update_post_meta($post->ID, '_stock_status', 'instock');
							//echo $theid . ' - ID - - instock<br /> ';
						} else {
							update_post_meta($post->ID, '_stock_status', $stockStatus);
							//echo $theid . ' - ID - - ' . $stockStatus . '<br /> ';
						}
					} else {
						
						if (!empty($map)) {
							foreach ($map as $meta => $internalId) {
								$internalId = self::formatLocationId($internalId);
								$quantity = 0;
								if ( ! empty( $inventory['locations'] ) ) {
									foreach ( $inventory['locations'] as $key => $loc ) {
										if ( is_array( $internalId ) && in_array( $loc['internalId'], $internalId ) ) {
											$quantity += $loc['quantity'];
										}
										if ( $loc['internalId'] == $internalId ) {
											$quantity = $loc['quantity'];
										}
									}
								} elseif ( isset( $inventory['quantity'] ) ) {
									$quantity = $inventory['quantity'];
								}
								$quantity = apply_filters('in8_inventory_quantity', $quantity, $inventory, $post->ID);
//								$quantity = $inventory['locations'][$locationKey]['quantity'];
								
								// stock
								update_post_meta($post->ID, $meta, $quantity);
								//echo $theid . ' - ID - ' . $quantity . 'Qty Updated<br /> ';
								
								if ($meta == '_stock') {
									if ($options['update_stock_status'] == 'yes') {
										$stockStatus = 'outofstock';
										$threshHold = (int) get_option( 'woocommerce_notify_no_stock_amount', 0 );
										
										if ($quantity > $threshHold) {
											$stockStatus = 'instock';
										}
										
										if ($options['backorders'] != 'disable') {
											if( in_array($options['backorders'], ['yes', 'notify']) ){
												$stockStatus = 'instock';
											}
										}
										
										if ($posttype == 'variable') {
											update_post_meta($post->ID, '_stock_status', 'instock');
											//echo $theid . ' - ID - - instock<br /> ';
										} else {
											update_post_meta($post->ID, '_stock_status', $stockStatus);
											//echo $theid . ' - ID - - ' . $stockStatus . '<br /> ';
										}
										//debug only
										//echo $internalId.", ".$post->ID.", ".$quantity.", ".$stockStatus.", ".$posttype.", ".$options['backorders']."<br>";
									}
									if ($options['update_stock_status'] == 'no') {
										$stockStatus = 'instock';
										update_post_meta($post->ID, '_stock_status', $stockStatus);
									}
									
									if ($options['backorders'] != 'disable') {
										// backorders
										update_post_meta($post->ID, '_backorders', $options['backorders']);
									}
									//echo $theid . ' - ID - ' . $options['backorders'] . ' Backorder status<br /> ';
								}
							}
						}
					}
					
					$request_stats['updated']++;
				}
			}
			$result_message = "Inventory update finished.<br />";
				$result_message .= "{$request_stats['updated']} WC products have been updated.";
				$result_message .= !empty($request_stats['inventories_missing']) ? "<br/>{$request_stats['inventories_missing']} products without inventories on NS." : "";
				$result_message .= !empty($request_stats['invalid_products']) ? "<br/>{$request_stats['invalid_products']} invalid products (wrong post type or parent type)." : "";
			$netsuite_connection->processMessage($result_message);
			$netsuite_connection->processMessage('Inventory sync completed', true);
			
			do_action( 'in8sync_inventory_complete' );
        }

        static function getProductsByInternalId() {
            global $wpdb;
            $options = self::options();
            $products = $wpdb->get_results("SELECT ID FROM {$wpdb->posts} WHERE (post_type='product'AND post_status='publish') OR (post_type='product_variation' AND post_status= 'publish' AND post_parent IN (SELECT ID FROM {$wpdb->posts} WHERE post_status= 'publish'))");
            
            $_products = [];
            foreach ($products as $post) {
                $theid = $post->ID;
                //$sku = get_post_meta($theid, '_sku', true );
                $ptype = get_post_type($theid);
                $stockst = get_post_meta($theid, '_stock_status', true);
                $pststatus = get_post_status($theid);
                //$product = new WC_Product($theid);

                if ($ptype == 'product_variation') {
                    $parent_id = wp_get_post_parent_id($theid);
					if( $parent_id > 0 ){
						update_post_meta($parent_id, '_stock_status', 'instock');
					}
                    if ($parent_id == 0) {
                        continue;
                    }
                }
				if ($options['update_stock_status'] == 'no') {
						$stockStatus = 'instock';					
						update_post_meta($theid, '_stock_status', $stockStatus);				
				}
                $internalid = get_post_meta($theid, 'internalid', true);

                //if ($sku == '') {continue;}
                if ($internalid == '') {
                    continue;
                }
                if ($internalid == 'Array') {
                    continue;
                }
                if ($stockst == '') {
                    continue;
                }
                if ($pststatus == '') {
                    continue;
                }

				if( isset($_products[$internalid]) && is_array($_products[$internalid]) ){
					$_products[$internalid][] = $post;
				}else{// if it's not an array then we create the array with the old value and new value
					$_products[$internalid] = [
						$post // new value
					];
				}
            }

            return apply_filters('in8_inventory_products_with_internalId', $_products);
        }

        static function getInventoriesFromNetsuite() {
            $url_options['script'] = "customscript_in8_inventory_rl";
            $options = self::options();

            // If selecting to return inventory from all locations, do not send the location to NetSuite
            if (isset($options['use_all_location_inventory']) && $options['use_all_location_inventory'] == 'on') {
                $url_options['searchType'] = 1;
            } else {
                if (!empty($options['location_id'])) {
	                $url_options['location'] = self::formatLocationId($options['location_id'], true);
                }
                if( !empty($options['multy_location_id']) ){
                	$mappings = self::getLocationMap();
	                $array = [];
	                
                	foreach($mappings as $val){
                		if( is_array($val) ) {
			                $array = array_unique(array_merge( $array, $val ));
		                }else{
                			$array[] = $val;
		                }
	                }
	                $url_options['location'] = self::formatLocationId(implode('+', $array), true);
                }                
            }
            // Filter by subsidiary
            if (!empty($options['subsidiary_id'])) {
                $url_options['subsidiary'] = $options['subsidiary_id'];
            }

            if (!class_exists('NetsuiteConnection')) { //in case of ajax
                //netsuite api connection file
                require_once(WP_PLUGIN_DIR . '/in8sync/code/netsuiteconnection.php');
            }

            $netsuite_connection = new NetsuiteConnection(0, 'inventory');
            $url_options = apply_filters('in8_inventory_request', $url_options, $options);
            $netsuite_connection_resp = $netsuite_connection->doRequest(array(), $url_options);
            
			do_action('in8_inventory_inventories_from_netsuite_request', $netsuite_connection_resp);
            
			$newInventories = [];
			if (isset($netsuite_connection_resp['success']) && $netsuite_connection_resp['success'] == 1 && !empty($netsuite_connection_resp['data'])) {
                $inventories = $netsuite_connection_resp['data'];
                // make the array use the internalid as key
                foreach ($inventories as $key => $inventory) {
                    $newInventories[$inventory['internalId']] = $inventory;
                    unset($inventories[$key]);
                }
            }
			return apply_filters('in8_inventory_inventories_from_netsuite_array', $newInventories);
		}

		static function formatLocationId($id, $for_ns = false){
        	if( preg_match('/\+/', $id) ){
		        $array = explode('+', $id);
		        if( $for_ns ){
		        	$array = '['.implode(',',$array).']';
		        }
        		return $array;
	        }
	        
	        return trim($id);
		}

        static function getLocationMap() {
            $options = self::options();

            $lId = $options['location_id'];
            $map = $options['multy_location_id'];

            if (empty($lId)) {
                $lId = [];
                if (empty($map)) {
                    $lId = ['_stock' => 0];
                } else {
                    if (!preg_match('/,/', $map)) {
                        $lId = ['_stock' => 0];
                    }

                    if (!preg_match('/=>/', $map)) {
                        $lId = ['_stock' => 0];
                    }

                    $result = explode(',', $map);
                    foreach ($result as $item) {
                        $wrap = explode('=>', $item);
	                    $locationInternalId = self::formatLocationId($wrap[0]);
                        if (count($wrap) == 1) {
                            $lId['_stock'] = $locationInternalId;
                        } else {
                            $lId[trim($wrap[1])] = $locationInternalId;
                        }
                    }
                }
            } else {
                $lId = ['_stock' => $lId];
            }

            return $lId;
        }

        public function getInternalIdsFromNS() {
            
            $locations = $subsidiaries = [];

            // Get a list of Locations
            $netsuite_connection = new NetsuiteConnection('0', 'location');
            $netsuite_connection_resp = $netsuite_connection->doRequest();

            if (isset($netsuite_connection_resp['success']) && $netsuite_connection_resp['success'] == 1 && !empty($netsuite_connection_resp['data'])) {
                $locations = $netsuite_connection_resp['data'];
            }

            // Get a list of Subsidiaries
            $netsuite_connection = new NetsuiteConnection('0', 'subsidiary');
            $netsuite_connection_resp = $netsuite_connection->doRequest();

            if (isset($netsuite_connection_resp['success']) && $netsuite_connection_resp['success'] == 1 && !empty($netsuite_connection_resp['data'])) {
                $subsidiaries = $netsuite_connection_resp['data'];
            }

            // Store as options
            $options = array();
            $options['locations'] = $locations;
            $options['subsidiaries'] = $subsidiaries;

            self::saveOptions($options);

            $template = self::getInternalIdsFromNSTemplate($locations, $subsidiaries);

            die($template);
        }

        public function getInternalIdsFromNSTemplate($locations, $subsidiaries) {

            $return_string = "";

            if (isset($locations) && !empty($locations)) {
                $return_string .= '<div class="well"><table class="table table-responsive table-bordered">';
                $return_string .= '<caption><h4>NetSuite Locations</h4></caption>';
                $return_string .= '<thead class="warning">';
                $return_string .= '<th>Name</th>';
                $return_string .= '<th>Internal ID</th></thead><tbody>';

                foreach ($locations as $location) {
                    $return_string .= '<tr><td>' . $location['name'] . '</td><td>' . $location['internalId'] . '</td></tr>';
                }
                $return_string .= '</tbody></table></div>';
            } else {
                $return_string .= '<div class="well">Error loading custom forms from NetSuite. Please check the error logs.</div>';
            }        

            if (isset($subsidiaries) && !empty($subsidiaries)) {
                $return_string .= '<div class="well"><table class="table table-responsive table-bordered">';
                $return_string .= '<caption><h4>NetSuite Subsidiaries</h4></caption>';
                $return_string .= '<thead class="warning">';
                $return_string .= '<th>Name</th>';
                $return_string .= '<th>Internal ID</th></thead><tbody>';

                foreach ($subsidiaries as $subsidiary) {
                    $return_string .= '<tr><td>' . $subsidiary['name'] . '</td><td>' . $subsidiary['internalId'] . '</td></tr>';
                }
                $return_string .= '</tbody></table></div>';
            } else {
                $return_string .= '<div class="well">Error loading subsidiaries from NetSuite. Please check the error logs.</div>';
            }                            

            return $return_string;
        }

        /**
         * @param type $options - array of all options
         * @return array of all options
         */
        public static function saveOptions($options = []) {
            
            $option_tag = 'In8_Inventory_NS';
            update_option($option_tag, $options);
            return $options;
        }

        public static function getOptions() {
            
            $option_tag = 'In8_Inventory_NS';

            $options = get_option($option_tag, []);
            if (empty($options)) {
                $options = [];
            }

            return $options;
        }
	
	
		private static function getNetsuiteConnection() {
			if (!class_exists('NetsuiteConnection')) { //in case of ajax
				//netsuite api connection file
				require_once(WP_PLUGIN_DIR . '/in8sync/code/netsuiteconnection.php');
			}
		
			//netsuite connection object
			return new NetsuiteConnection(0, 'inventory');
		}
    }

    $In8_NetSuite_Inventory = new In8_NetSuite_Inventory();
    